<?php # $Id: movabletype.inc.php,v 1.5 2005/01/26 14:14:05 garvinhicking Exp $
# Copyright (c) 2003-2005, Jannis Hermanns (on behalf the Serendipity Developer Team)
# All rights reserved.  See LICENSE file for licensing details

/*****************************************************************
 *                MovableType Importer, by Evan Nemerson         *
 *****************************************************************/

switch ($serendipity['lang']) {
    case 'de':
        @define('IMPORTER_MT_WARN_PLUGIN',     'Bitte installieren Sie das Plugin "%s"');
        @define('IMPORTER_MT_NOTE', 'Falls Sie weiter machen, ohne die Plugins zu installieren, werden mglicherweise Zeilenumbrche falsch importiert (verdoppelt oder entfernt)');
        break;

    case 'en':
    default:
        @define('IMPORTER_MT_WARN_PLUGIN',     'Please install the plugin "%s"');
        @define('IMPORTER_MT_NOTE', 'If you continue without installing those plugins, line breaks may be incorrectly imported (doubled or removed)');
        break;
}

class Serendipity_Import_MovableType extends Serendipity_Import {
    var $info        = array('software' => 'MovableType');
    var $data        = array();
    var $inputFields = array();

    function Serendipity_Import_MovableType($data) {
        $this->data = $data;
        $this->inputFields = array(array('text'    => MT_DATA_FILE,
                                         'type'    => 'file',
                                         'name'    => 'mt_dat'),

                                   array('text'    => FORCE,
                                         'type'    => 'bool',
                                         'name'    => 'mt_force',
                                         'default' => 'true'),

                                   array('text'    => ACTIVATE_AUTODISCOVERY,
                                         'type'    => 'bool',
                                         'name'    => 'autodiscovery',
                                         'default' => 'false')
                            );
    }

    function getImportNotes(){
        $notes = array();
        if (!class_exists('serendipity_event_nl2br')){
            $notes[] = sprintf(IMPORTER_MT_WARN_PLUGIN, 'serendipity_event_nl2br');
        }
        if (!class_exists('serendipity_event_entryproperties')){
            $notes[] = sprintf(IMPORTER_MT_WARN_PLUGIN, 'serendipity_event_entryproperties');
        }
        if (count($notes) > 0){
            return '<ul><li>'.implode('</li><li>', $notes).'</li></ul>'.IMPORTER_MT_NOTE;
        }
    }
    function validateData() {
        return sizeof($this->data);
    }

    function getInputFields() {
        return $this->inputFields;
    }

    function doEntryWork(&$mt_entry, &$tasks){
        $entry = array();
        $entry['categories'] = array();
        $entryprops = array();

        foreach($mt_entry as $name => $data){
            switch($name){
            case 'AUTHOR':
                    if ( !isset($authors[$data]) ) {
                        $au_inf = serendipity_fetchAuthor($data);
                        if ( !is_array($au_inf) ) {
                            $tasks[] = sprintf(CREATE_AUTHOR, htmlspecialchars($data));
                            $au_inf = serendipity_fetchAuthor($serendipity['authorid']);
                        }
                        $authors[$data] = $au_inf[0];
                    }
                    $entry['authorid'] = $authors[$data]['authorid'];
                    $entry['author'] = $authors[$data]['username'];
                    break;
                case 'TITLE':
                    $entry['title'] = $data;
                    break;
                case 'STATUS':
                    $entry['isdraft'] = ($data == 'Publish') ? 'false' : 'true';
                    break;
                case 'ALLOW COMMENTS':
                    $entry['allow_comments'] = ($data == '1') ? 'true' : 'false';
                    break;
                case 'DATE':
                    $entry['timestamp'] = strtotime($data);
                    if ( $entry['timestamp'] == 0 ) {
                        $entry['timestamp'] = time();
                    }
                    break;
                case 'BODY':
                    $entry['body'] = $data;
                    break;
                case 'EXTENDED BODY':
                    $entry['extended'] = $data;
                    break;
                case 'CONVERT BREAKS':
                    $entryprops['nl2br'] = ($data == '1') ? true : false;
                    break;
                case 'PRIMARY CATEGORY':
                case 'CATEGORY':
                      $cat_found = false;
                      if ( is_array($this->categories) ) {
                          for ( $y=0 ; $y<sizeof($this->categories) ; $y++ ) {
                              if ( $this->categories[$y]['category_name'] == $data ) {
                                  $cat_found = true;
                                  break;
                              }
                          }
                          if ( $cat_found){
                              if (!in_array($this->categories[$y]['categoryid'], $entry['categories']) ) {
                                  //$entries[$n]['categories'][] = $categories[$y]['categoryid'];
                                  $entry['categories'][] = $this->categories[$y]['categoryid'];
                              }
                          }else {
                              $tasks[] = sprintf(CREATE_CATEGORY, htmlspecialchars($data));
                          }
                      }
                      break;
            }
        }
        $entry['props'] = $entryprops;
        return $entry;
    }

    function import() {
        global $serendipity;

        $force    = ($this->data['force'] == 'true');

        if ($this->data['autodiscovery'] == 'false') {
            $serendipity['noautodiscovery'] = 1;
        }

        // Rewritten to parse the file line by line. Can save quite some
        // memory on large blogs
        //$contents   = file_get_contents($_FILES['serendipity']['tmp_name']['import']['mt_dat']);

        $authors    = array();
        $this->categories = serendipity_fetchCategories();
        $tasks      = array();

        $entries = array();

        $fh = fopen($_FILES['serendipity']['tmp_name']['import']['mt_dat'], 'r');

        $n = 0;

        $entry = array();
        $el    = "";
        $skip  = false;
        while (!feof($fh)){
            $line = fgets($fh, 8192);
            if ($skip && (!preg_match('/^--------/', $line)))
                continue;
            if (preg_match('/^--------/', $line)){
                // We found the end marker of the current entry. Add to
            // entries-Array
                $entries[] = $this->doEntryWork($entry, $tasks);
                $entry = array();
                $skip = false;
            }elseif (preg_match('/^-----/', $line)){
                if (empty($el)){
                    $line = fgets($fh, 8192);
                    if (preg_match('/^([A-Z]+):/', $line, $matches)){
                        if ($matches[1] == 'COMMENT'){
                            // todo: really handle comments.
                           $skip = true;
                        }else{
                           $el = $matches[1];
                        }
                    }
                }else{
                    $el = "";
                }
           }else{
               if (empty($el) ){
                   if (preg_match('/^([A-Z ]+): (.*)$/', $line, $matches)){
                       $entry[$matches[1]] = $matches[2];
                   }
               }else{
                   $entry[$el] .= $line;
               }
           }

        }
        fclose($fh);

        if ( !sizeof($tasks) || $force == true ) {
            serendipity_db_begin_transaction();
            foreach ( $entries as $entry ) {
                $entryprops = $entry['props'];
                unset($entry['props']);
                if ( !is_int($r = serendipity_updertEntry($entry)) ) {
                    echo '<div class="serendipityAdminMsgError">' . $r . '</div>';
                }else{
                    $entry['id'] = $r;
                    // Let the plugins do some additional stuff. Here it's used with
                    // event_entryproperties in mind to setup the nl2br-stuff
                    serendipity_plugin_api::hook_event('backend_import_entry', $entry, $entryprops);
                }
            }
            serendipity_db_end_transaction(true);
            return true;
        }else {
            return '<ul><li>'.implode('</li><li>', array_unique($tasks)).'</li></ul>';
        }
    }
}
return 'Serendipity_Import_MovableType';
?>
